/**
 * Represents a chunk of text with its associated metadata.
 */
export type TextChunk = {
    /**
     * Unique identifier for the text chunk.
     */
    id: string;
    
    /**
     * The actual text content of the chunk.
     */
    text: string;
    
    /**
     * Optional number of tokens in the input text.
     */
    inputTokens?: number;
    
    /**
     * Optional vector embeddings for the text chunk.
     */
    embeddings?: number[];
};

/**
 * Result from an embedding operation.
 */
export type EmbeddingResult = {
    /**
     * Optional count of tokens in the input text.
     */
    inputTokens?: number;
    
    /**
     * Vector representation of the input text.
     */
    embedding: number[];
};

/**
 * Abstract class that defines text splitting functionality.
 */
export abstract class TextSplitter {
    /**
     * Creates a new TextSplitter instance.
     * 
     * @param chunkSize - Maximum size of each text chunk.
     * @param chunkOverlap - Number of overlapping characters between chunks.
     */
    constructor(
        chunkSize?: number,
        chunkOverlap?: number,
    );

    /**
     * Splits a text into multiple chunks.
     * 
     * @param text - The text to split.
     * @returns An array of TextChunk objects.
     */
    abstract split(text: string): TextChunk[];
}

/**
 * A concrete implementation of TextSplitter that splits text by sentences.
 */
export class SentenceSplitter extends TextSplitter {
    /**
     * Splits a text into multiple chunks by sentences.
     * 
     * @param text - The text to split.
     * @returns An array of TextChunk objects.
     */
    split(text: string): TextChunk[];
}

/**
 * Base parameters for embedding operations.
 */
export type BaseEmbeddingAdditionalParams = {
    /**
     * Number of parallel processing operations to perform.
     */
    parallelProcessing?: number;
};

/**
 * Abstract base class for text embedding functionality.
 * 
 * @typeParam TAdditionalParams - Type of additional parameters for embedding operations.
 */
export abstract class BaseEmbedding<
    TAdditionalParams extends BaseEmbeddingAdditionalParams,
> {
    /**
     * Creates a new BaseEmbedding instance.
     * 
     * @param textSplitter - The text splitter to use for chunking text.
     */
    protected constructor(textSplitter: TextSplitter);

    /**
     * Gets embeddings for a text.
     * 
     * @param text - The text to embed.
     * @param additionalParams - Additional parameters for the embedding operation.
     * @returns A promise resolving to an EmbeddingResult.
     */
    protected abstract getEmbedding(
        text: string,
        additionalParams?: TAdditionalParams,
    ): Promise<EmbeddingResult>;

    /**
     * Embeds a text by splitting it into chunks and generating embeddings for each chunk.
     * 
     * @param text - The text to embed.
     * @param additionalParams - Additional parameters for the embedding operation.
     * @returns a promise resolving to an array of TextChunk objects with embeddings.
     */
    embedText(
        text: string,
        additionalParams?: TAdditionalParams,
    ): Promise<TextChunk[]>;
}