import { BedrockEmbedding } from './bedrock-embedding';
import {
    BaseEmbeddingAdditionalParams,
    EmbeddingResult,
} from './base-embedding';

/**
 * Additional parameters for the Amazon Titan Embed Text V2 model.
 * Extends the base embedding parameters with Titan-specific options.
 * 
 * Properties:
 * - dimensions?: The dimension size of the output embedding vector. 
 *   Possible values are 256, 512, or 1024 (default: 1024).
 * - normalize?: Whether to normalize the embedding vectors (default: true).
 *   When true, each embedding vector will have a Euclidean norm of 1.
 */
export declare type AmazonTitanEmbedTextV2AdditionalParams =
    BaseEmbeddingAdditionalParams & {
        dimensions?: 256 | 512 | 1024;
        normalize?: boolean;
    };

/**
 * Implementation of the Amazon Titan Embedding model for converting text into vector embeddings.
 * This class uses Amazon Bedrock's Titan Embed Text V2 model to generate numerical representations
 * of text that capture semantic meaning.
 * 
 * These embeddings can be used for:
 * - Semantic search
 * - Text similarity comparison
 * - Content recommendation
 * - And other NLP tasks requiring vector representations
 * 
 * @extends BedrockEmbedding<AmazonTitanEmbedTextV2AdditionalParams>
 */
export declare class TitanEmbedding extends BedrockEmbedding<AmazonTitanEmbedTextV2AdditionalParams> {
    /**
     * Generates an embedding vector for the provided text using the Amazon Titan model.
     * 
     * @param text - The input text to be converted into an embedding vector
     * @param additionalParams - Optional parameters to customize the embedding generation
     * @returns A Promise resolving to an EmbeddingResult containing the embedding vector and input token count
     */
    protected getEmbedding(
        text: string,
        additionalParams?: AmazonTitanEmbedTextV2AdditionalParams,
    ): Promise<EmbeddingResult>;
}