import { ConverseCommandInput } from '@aws-sdk/client-bedrock-runtime';
import { BedrockLlmModel } from './bedrock-llm-model';
import {
    LlmInterface,
    LlmCallParameters,
    LlmMessage,
    LLmResponse,
    LLmResponseContent,
} from '../../llm.interface';

/**
 * Type representing possible role types for Bedrock messages.
 */
export type BedrockMessageRoleType = 'user' | 'assistant' | 'system';

/**
 * Type representing supported image formats for Bedrock content.
 */
export type BedrockImageFormat = 'png' | 'jpeg';

/**
 * Type representing text content that can be sent to Bedrock API.
 */
export type BedrockContentText = {
    text: string;
};

/**
 * Type representing image content that can be sent to Bedrock API.
 */
export type BedrockContentImage = {
    image: {
        format: BedrockImageFormat;
        source: { bytes: Buffer };
    };
};

/**
 * Union type representing possible content types in Bedrock requests.
 */
export type BedrockContent = BedrockContentText | BedrockContentImage;

/**
 * Type representing a message to be sent to Bedrock API.
 */
export type BedrockMessage = {
    /**
     * Role of the entity sending the message.
     */
    role: BedrockMessageRoleType;
    /**
     * Array of content items in the message.
     */
    content: BedrockContent[];
};

/**
 * Type representing a text response type from Bedrock.
 */
export type BedrockResponseTypeText = 'text';

/**
 * Type representing a reasoning response type from Bedrock.
 */
export type BedrockResponseTypeReasoning = 'reasoning';

/**
 * Type representing an unknown response type from Bedrock.
 */
export type BedrockResponseTypeUnknow = 'unknow';

/**
 * Type representing a usage metadata response type from Bedrock.
 * This type is used to indicate content that contains usage statistics
 * such as token counts and API consumption data.
 */
export type BedrockResponseTypeUsage = 'usage';

/**
 * Type representing text content in Bedrock response.
 */
export type BedrockResponseContentText = {
    /**
     * Type of the response content.
     */
    type: BedrockResponseTypeText;
    /**
     * Text content of the response.
     */
    data: string;
};

/**
 * Type representing reasoning content in Bedrock response.
 */
export type BedrockResponseContentReasoning = {
    /**
     * Type of the response content.
     */
    type: BedrockResponseTypeReasoning;
    /**
     * Reasoning content as text.
     */
    data: string;
};

/**
 * Type representing unknown content in Bedrock response.
 */
export type BedrockResponseContentUnknow = {
    /**
     * Type of the response content.
     */
    type: BedrockResponseTypeUnknow;
    /**
     * Unknown content, either as string or another type.
     */
    data: string | unknown;
};

/**
 * Type representing usage metadata content in Bedrock response.
 * The content is a string representing a JSON object in the format:
 * {
 *     inputTokens: number;
 *     outputTokens: number;
 * }
 */
export type BedrockResponseContentUsage = {
    /**
     * Identifies this as usage statistics content
     */
    type: BedrockResponseTypeUsage;
    /**
     * The usage statistics content as a JSON string containing inputTokens and outputTokens
     */
    data: string;
};

/**
 * Union type representing possible content types in Bedrock response.
 */
export type BedrockResponseContent =
    | BedrockResponseContentText
    | BedrockResponseContentReasoning
    | BedrockResponseContentUnknow
    | BedrockResponseContentUsage;

/**
 * Type representing a complete response from Bedrock API.
 */
export type BedrockResponse = {
    /**
     * Usage statistics for the API call.
     */
    usage: {
        /**
         * Number of tokens in the input.
         */
        inputTokens: number;
        /**
         * Number of tokens in the output.
         */
        outputTokens: number;
    };
    /**
     * Array of response content items.
     */
    content: BedrockResponseContent[];
};

/**
 * Abstract class for interacting with AWS Bedrock API.
 * Implements the LlmInterface for standardized language model interactions.
 *
 * @template TAdditionalParams Type for additional parameters specific to the implementation.
 */
export abstract class BedrockLlm<TAdditionalParams>
    implements LlmInterface<TAdditionalParams>
{
    /**
     * Creates a new instance of BedrockLlm.
     *
     * @param accessKeyId AWS access key ID for authentication.
     * @param secretAccessKey AWS secret access key for authentication.
     * @param region AWS region where the Bedrock service is deployed.
     * @param model Bedrock model to use for API calls.
     */
    constructor(
        accessKeyId: string,
        secretAccessKey: string,
        region: string,
        model: BedrockLlmModel,
    );

    /**
     * Gets the maximum number of tokens supported by the model.
     *
     * @returns The maximum number of tokens.
     */
    protected getMaxTokens(): number | undefined;

    /**
     * Creates a request object for the Bedrock API.
     *
     * @param messages Array of messages to send to the model.
     * @param temperature Controls randomness in the output. Default: 0.5
     * @param maxTokens Maximum number of tokens to generate. Default: null
     * @param systemPrompt Optional system prompt to guide the model's behavior. Default: undefined
     * @param topP Controls diversity via nucleus sampling. Default: null
     * @param additionalArgs Additional model-specific arguments. Default: undefined
     * @returns A configured ConverseCommandInput object.
     */
    protected createRequest(
        messages: BedrockMessage[],
        temperature?: number,
        maxTokens?: number,
        systemPrompt?: string,
        topP?: number,
        additionalArgs?: TAdditionalParams,
    ): ConverseCommandInput;

    /**
     * Invokes the Bedrock API with the provided parameters.
     *
     * @param messages Array of messages to send to the model.
     * @param temperature Controls randomness in the output. Default: 0.5
     * @param maxTokens Maximum number of tokens to generate. Default: null
     * @param systemPrompt Optional system prompt to guide the model's behavior. Default: undefined
     * @param topP Controls diversity via nucleus sampling. Default: undefined
     * @param additionalArgs Additional model-specific arguments. Default: undefined
     * @returns Promise resolving to a BedrockResponse object.
     */
    public invoke(
        messages: BedrockMessage[],
        temperature?: number,
        maxTokens?: number,
        systemPrompt?: string,
        topP?: number,
        additionalArgs?: TAdditionalParams,
    ): Promise<BedrockResponse>;

    /**
     * Invokes the Bedrock API in streaming mode with the provided parameters.
     *
     * @param messages Array of messages to send to the model
     * @param temperature Controls randomness in the output. Default: 0.5
     * @param maxTokens Maximum number of tokens to generate. Default: null
     * @param systemPrompt Optional system prompt to guide the model's behavior. Default: undefined
     * @param topP Controls diversity via nucleus sampling. Default: undefined
     * @param additionalArgs Additional model-specific arguments. Default: undefined
     * @returns AsyncGenerator that yields BedrockResponseContent chunks
     */
    public invokeStream(
        messages: BedrockMessage[],
        temperature?: number,
        maxTokens?: number | null,
        systemPrompt?: string,
        topP?: number,
        additionalArgs?: TAdditionalParams,
    ): AsyncGenerator<BedrockResponseContent, void, unknown>;

    /**
     * Calls the language model using the standardized LlmInterface method.
     *
     * @param messages Array of messages in the standardized format.
     * @param systemPrompt Optional system prompt to guide the model's behavior. Default: undefined
     * @param parameters Optional parameters for the call, including temperature, maxTokens,
     *                   topP, and additionalParams. Default: undefined
     * @returns Promise resolving to a standardized LLmResponse object.
     */
    public callLlm(
        messages: LlmMessage[],
        systemPrompt?: string,
        parameters?: LlmCallParameters<TAdditionalParams>,
    ): Promise<LLmResponse>;

    /**
     * Calls the LLM with a set of messages and parameters, returning a stream of responses.
     * @param messages - Array of messages representing the conversation history
     * @param systemPrompt - Optional system prompt to guide the model's behavior
     * @param parameters - Optional parameters to control the model's output
     * @returns AsyncGenerator yielding incremental responses from the LLM
     */
    callLlmStream(
        messages: LlmMessage[],
        systemPrompt?: string,
        parameters?: LlmCallParameters<TAdditionalParams>,
    ): AsyncGenerator<LLmResponseContent, void, unknown>;
}
