import { Buffer } from 'node:buffer';

/**
 * Represents the type of data content in a message.
 */
export type LlmMessageTypeText = 'text';

/**
 * Represents the type of image content in a message.
 */
export type LlmMessageTypeImage = 'image';

/**
 * Defines the possible roles in a conversation with an LLM.
 */
export type LlmMessageRoleType = 'user' | 'assistant' | 'system';

/**
 * Structure for data content within a message.
 */
export type LlmMessageContentText = {
    /** Identifies this content as data */
    type: LlmMessageTypeText;
    /** The actual data content */
    text: string;
};

/**
 * Supported formats for images.
 */
export type LlmImageFormat = 'png' | 'jpeg';

/**
 * Structure for image content within a message.
 */
export type LlmMessageContentImage = {
    /** Identifies this content as an image */
    type: LlmMessageTypeImage;
    /** The format of the image */
    format: LlmImageFormat;
    /** Binary data of the image */
    data: Buffer;
};

/**
 * Union type representing all possible message content types.
 */
export type LlmMessageContent = LlmMessageContentText | LlmMessageContentImage;

/**
 * Structure of a message in a conversation with an LLM.
 */
export type LlmMessage = {
    /** The role of the entity sending the message */
    role: LlmMessageRoleType;
    /** Array of content components that make up the message */
    contents: LlmMessageContent[];
};

/**
 * Type for reasoning content in responses.
 */
export type LlmResponseTypeReasoning = 'reasoning';

/**
 * Type for unknown or unspecified content in responses.
 */
export type LlmResponseTypeUnknow = 'unknow';

/**
 * Type for data content in responses.
 */
export type LlmResponseTypeText = 'text';

/**
 * Type for usage statistics content in responses.
 */
export type LlmResponseTypeUsage = 'usage';

/**
 * Structure for reasoning content in a response.
 */
export type LLmResponseContentReasoning = {
    /** Identifies this as reasoning content */
    type: LlmResponseTypeReasoning;
    /** The reasoning content */
    content: string;
};

/**
 * Structure for data content in a response.
 */
export type LLmResponseContentText = {
    /** Identifies this as data content */
    type: LlmResponseTypeText;
    /** The data content */
    content: string;
};

/**
 * Structure for unknown or unspecified content in a response.
 */
export type LLmResponseContentUnknow = {
    /** Identifies this as unknown content */
    type: LlmResponseTypeUnknow;
    /** The content with an undefined structure */
    content: string | unknown;
};

/**
 * Structure for usage statistics content in a response.
 * The content is a string representing a JSON object in the format:
 * {
 *     inputTokens: number;
 *     outputTokens: number;
 * }
 */
export type LLmResponseContentUsage = {
    /** Identifies this as usage statistics content */
    type: LlmResponseTypeUsage;
    /** The usage statistics content as a JSON string containing inputTokens and outputTokens */
    content: string;
};

/**
 * Union type representing all possible response content types.
 */
export type LLmResponseContent =
    | LLmResponseContentReasoning
    | LLmResponseContentText
    | LLmResponseContentUnknow
    | LLmResponseContentUsage;

/**
 * Information about token usage in an LLM request/response.
 */
export type LLmResponseUsage = {
    /** Number of tokens in the input/request */
    inputTokens: number;
    /** Number of tokens in the output/response */
    outputTokens: number;
};

/**
 * Structure of a complete response from an LLM.
 */
export type LLmResponse = {
    /** Token usage statistics */
    usage: LLmResponseUsage;
    /** Array of content components that make up the response */
    contents: LLmResponseContent[];
};

/**
 * Parameters to control the behavior of the LLM.
 * @template TAdditionalParams - Type for model-specific additional parameters
 */
export type LlmCallParameters<TAdditionalParams> = {
    /** Controls randomness in output (higher = more random) */
    temperature?: number;
    /** Maximum number of tokens to generate */
    maxTokens?: number;
    /** Controls diversity via nucleus sampling */
    topP?: number;
    /** Additional model-specific parameters */
    additionalParams?: TAdditionalParams;
};

/**
 * Interface for interacting with language learning models (LLMs).
 * @template TAdditionalParams - Type for model-specific additional parameters
 */
export interface LlmInterface<TAdditionalParams> {
    /**
     * Calls the LLM with a set of messages and parameters.
     * @param messages - Array of messages representing the conversation history
     * @param systemPrompt - Optional system prompt to guide the model's behavior
     * @param parameters - Optional parameters to control the model's output
     * @returns Promise resolving to the LLM's response
     */
    callLlm(
        messages: LlmMessage[],
        systemPrompt?: string,
        parameters?: LlmCallParameters<TAdditionalParams>,
    ): Promise<LLmResponse>;

    /**
     * Calls the LLM with a set of messages and parameters, returning a stream of responses.
     * @param messages - Array of messages representing the conversation history
     * @param systemPrompt - Optional system prompt to guide the model's behavior
     * @param parameters - Optional parameters to control the model's output
     * @returns AsyncGenerator yielding incremental responses from the LLM
     */
    callLlmStream(
        messages: LlmMessage[],
        systemPrompt?: string,
        parameters?: LlmCallParameters<TAdditionalParams>,
    ): AsyncGenerator<LLmResponseContent, void, unknown>;
}

/**
 * Convenience type for an LLM interface with unknown additional parameters.
 */
export type Llm = LlmInterface<unknown>;
