import { LlmCallParameters, LlmImageFormat, LlmInterface } from '../../llm';

/**
 * Represents a divided image with orientation information and the resulting image parts.
 */
export type DividedImage = {
    /**
     * The orientation in which the image was divided.
     * 'vertical' means the original image was divided along vertical lines (landscape orientation).
     * 'horizontal' means the original image was divided along horizontal lines (portrait orientation).
     */
    orientation: 'vertical' | 'horizontal';

    /**
     * The format of the images (e.g., 'jpeg', 'png').
     */
    imageFormat: LlmImageFormat;

    /**
     * Array of image buffers representing the divided parts of the original image.
     */
    images: Buffer[];
};

/**
 * Converts a string format to a valid LLM image format.
 * Normalizes and validates the input format against supported formats.
 *
 * @param format - The image format string to convert (e.g., 'jpg', 'jpeg', 'png')
 * @returns The normalized LLM image format ('jpeg' or 'png')
 * @throws Error if the input format is not supported
 */
export function getLlmImageFormat(format: string): LlmImageFormat;

/**
 * Divides an image into proportional parts based on its aspect ratio.
 * If the image's aspect ratio is within the specified tolerance, the original image is returned unchanged.
 * For images with a significant aspect ratio difference, the function will divide it into 2-4 parts
 * to achieve a more balanced aspect ratio for each part.
 *
 * @param imageBuffer - The original image buffer to be divided.
 * @param tolerancePercent - The tolerance percentage for aspect ratio difference.
 *   If the difference between width and height is less than this percentage, the image won't be divided.
 *   Defaults to 20.
 * @returns A promise that resolves to a DividedImage object containing the
 *   divided image parts along with orientation information.
 */
export function divideImageIntoProportionalParts(
    imageBuffer: Buffer,
    tolerancePercent?: number,
): Promise<DividedImage>;

/**
 * Extracts markdown text from a set of divided images using a language model.
 * The images are processed as a continuous whole, as if they were stitched together.
 *
 * @template T - The type parameter for LLM-specific options.
 * @param llm - The language model interface to use for extraction.
 * @param dividedImages - The divided image parts to extract text from.
 * @param prompt - Optional custom prompt to guide the extraction process.
 *   If not provided, a default prompt is used that instructs the model to extract text in markdown format.
 * @param llmParameters - Optional parameters to configure the LLM call.
 * @returns A promise that resolves to the extracted markdown text.
 */
export function extractMarkdownFromDividedImages<T = unknown>(
    llm: LlmInterface<T>,
    dividedImages: DividedImage,
    prompt?: string,
    llmParameters?: LlmCallParameters<T>,
): Promise<string>;
