import { Buffer } from 'node:buffer';
import { LlmCallParameters, LlmImageFormat, LlmInterface } from '../../llm';

/**
 * Converts a PDF file buffer into an array of image buffers, where each image represents a page of the PDF.
 *
 * @param pdfBuffer - The buffer of the PDF file to be converted to images.
 * @param density - The resolution density (in DPI) to use for image conversion. Defaults to 150.
 * @param quality - The quality level (0-100) for the converted images. Defaults to 60.
 * @returns A promise that resolves to an array of image buffers, each representing a page of the PDF.
 */
export function convertPdfBufferToImages(
    pdfBuffer: Buffer,
    density?: number,
    quality?: number,
): Promise<Buffer[]>;


/**
 * Configuration options for PDF to markdown extraction.
 */
export type ExtractMarkdownFromPdfOptions<T = unknown> = {
    /**
     * The resolution density (DPI) for PDF to image conversion.
     * Uses the default value from convertPdfBufferToImages (150) if not specified.
     */
    density?: number;

    /**
     * The quality level for the converted images.
     * Uses the default value from convertPdfBufferToImages (60) if not specified.
     */
    quality?: number;

    /**
     * Options for image division.
     * Defaults to {enable: true, tolerance: 20} if not specified.
     */
    divideImageIntoProportionalParts?: {
        /**
         * Whether to enable image division.
         */
        enable: boolean;

        /**
         * Tolerance percentage for aspect ratio difference.
         * Defaults to 20.
         */
        tolerance?: number;
    };

    /**
     * Custom prompt to guide the extraction process.
     * Uses the default prompt from extractMarkdownFromDividedImages if not specified.
     */
    prompt?: string;

    /**
     * Parameters to configure the LLM call.
     */
    llmParameters?: LlmCallParameters<T>;

    /**
     * Maximum number of parallel processing operations.
     * Defaults to 2.
     */
    parallelProcessing?: number;

    /**
     * String to insert between extracted pages.
     * Defaults to "\n<!-- PAGE BREAK -->\n".
     */
    pageDivisor?: string;
};

/**
 * Extracts markdown text from a PDF buffer using a language model.
 * The function converts the PDF to images, optionally divides the images into proportional parts,
 * and then extracts text from each image using the provided language model.
 *
 * @template T - The type parameter for LLM-specific options.
 * @param llm - The language model interface to use for extraction.
 * @param pdfBuffer - The PDF file buffer to extract text from.
 * @param options - Configuration options for the extraction process.
 *   Defaults to an empty object, using all default options.
 * @returns A promise that resolves to the extracted markdown text from the entire PDF.
 *   Pages are separated by the specified pageDivisor (defaults to "\n<!-- PAGE BREAK -->\n").
 */
export function extractMarkdownFromPdf<T = unknown>(
    llm: LlmInterface<T>,
    pdfBuffer: Buffer,
    options?: ExtractMarkdownFromPdfOptions<T>,
): Promise<string>;