/**
 * A function type that returns a Promise of any type.
 */
export type PromiseFactory = () => Promise<unknown>;

/**
 * Executes an array of promise-returning functions with limited concurrency.
 *
 * @param tasks - An array of functions that return promises to be executed.
 * @param concurrency - The maximum number of promises to execute simultaneously.
 * @param stopOnError - Optional boolean that determines if execution should stop when an error occurs.
 *                      If true (default), the first error will reject the promise.
 *                      If false, errors will be included in the results array and execution will continue.
 * @param result - Optional array to store the results. If provided, the function will populate this exact array instance
 *                 with the resolved values from the tasks, maintaining the same order as the input tasks. If not provided,
 *                 a new array will be created with a length matching the tasks array.
 * @returns A promise that resolves to an array of results in the same order as the input tasks. If the `result` parameter
 *          was provided, the returned array will be the same instance.
 * @throws Error if concurrency is less than or equal to zero.
 *
 * @example
 * const tasks = [
 *   () => fetchData('url1'),
 *   () => fetchData('url2'),
 *   () => fetchData('url3')
 * ];
 *
 * // Execute with at most 2 simultaneous tasks
 * const results = await limitPromisesInParallel(tasks, 2);
 *
 * // Using a pre-existing array to store results
 * const myArray = [];
 * await limitPromisesInParallel(tasks, 2, true, myArray);
 * // myArray now contains the results, and is the same instance that was passed in
 *
 * // Continue execution even if some tasks fail
 * const resultsWithErrors = await limitPromisesInParallel(tasks, 2, false);
 * // Errors will be included in the results array instead of rejecting the promise
 */
export function limitPromisesInParallel(
    tasks: PromiseFactory[],
    concurrency: number,
    stopOnError?: boolean,
    result?: unknown[],
): Promise<unknown[]>;
